#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : ann_content
# @Time         : 2020/11/18 7:46 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.np_utils import normalize

from tql.nlp.utils.bert_utils import Bert
from bert4keras.snippets import sequence_padding

# Bert
bert = Bert()

tokenizer = bert.tokenizer
simbert_encoder = bert.simbert_encoder


def texts2vec(texts, is_lite=0, batch_size=1000, maxlen=64):
    X = []
    S = []
    for text in texts:
        token_ids, segment_ids = tokenizer.encode(text, maxlen=maxlen)
        X.append(token_ids)
        S.append(segment_ids)

    data = [sequence_padding(X, length=maxlen), sequence_padding(S, length=maxlen)]

    del text, X, S
    gc.collect()

    vecs = simbert_encoder.predict(data, batch_size=batch_size)

    if is_lite:
        vecs = vecs[:, range(0, 768, 4)]

    return normalize(vecs)


# p = Path('/fds/1_Work/3_ANN/date=20201117')
#
#
# def reader(p):
#     return pd.read_csv(p, '\t', names=['id', 'category', 'source', 'title', 'url'])
#
#
# df = (
#     pd.concat(p.glob("*") | xProcessPoolExecutor(reader, 8))
#         .dropna(subset=['id', 'category', 'title'])
#         .drop_duplicates('title')
#     [lambda df: df.title.str.len() >= 5][lambda df: df.title.str.len() <= 64]
#     [lambda df: ~df.id.str.contains('duokan')]
# )
# df['vector'] = texts2vec(tqdm(df['title'].astype(str))).tolist()
# df.to_hdf('title.h5', 'w')

df = pd.read_hdf('/fds/1_Work/3_ANN/title.h5').assign(title=lambda df: df['title'].astype(str))

df['vector'] = texts2vec(tqdm(df['title'])).tolist()

df.to_hdf('/fds/1_Work/3_ANN/title_vec.h5', 'w', complib='blosc', complevel=8)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-App.
# @File         : __init__.py
# @Time         : 2019-09-02 14:01
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

"""
https://github.com/Asmodius/sanic-scheduler
"""
import asyncio
import inspect
import logging
import traceback
from datetime import datetime, time, timedelta
from typing import Callable, Optional, Union

__all__ = ('task', 'SanicScheduler', 'make_task')

logger = logging.getLogger('scheduler')

_tasks = {}
_wrk = []


def make_task(fn: Callable,
              period: Optional[timedelta] = None,
              start: Optional[Union[timedelta, time]] = None) -> None:
    """Make task."""
    _tasks[fn] = Task(fn, period, start)


def task(period: Optional[timedelta] = None,
         start: Optional[Union[timedelta, time]] = None):
    """Decorate the function to run on schedule."""

    def wrapper(fn):
        make_task(fn, period, start)
        return fn

    return wrapper


class SanicScheduler:
    def __init__(self, app=None, utc=True):
        self.app = app
        if app:
            self.init_app(app, utc)

    def init_app(self, app, utc=True):
        self.app = app

        @app.listener("after_server_start")
        async def run_scheduler(_app, loop):
            for i in _tasks.values():
                _wrk.append(loop.create_task(i.run(_app, utc)))

        @app.listener("before_server_stop")
        async def stop_scheduler(_app, _):
            for i in _wrk:
                i.cancel()

        return self

    @classmethod
    def task_info(cls):
        return _tasks


class Task:
    def __init__(self,
                 func: Callable,
                 period: Optional[timedelta],
                 start: Optional[Union[timedelta, time]]):
        self.func = func
        self.func_name = func.__name__
        self.period = period
        self.start = start
        self.last_run = None

    def _next_run(self, utc):
        if utc:
            now = datetime.utcnow().replace(microsecond=0)
        else:
            now = datetime.now().replace(microsecond=0)

        if self.last_run is None:
            if self.start is not None:
                if isinstance(self.start, time):
                    d1 = datetime.combine(datetime.min, self.start)
                    d2 = datetime.combine(datetime.min, now.time())
                    self.start = timedelta(seconds=(d1 - d2).seconds)

                self.last_run = now + self.start
            else:
                self.last_run = now

        elif self.period is None:
            return

        else:
            while self.last_run <= now:
                self.last_run += self.period

        return self.last_run - now

    async def run(self, app, utc=True):
        while True:
            delta = self._next_run(utc)
            if delta is None:
                logger.info('STOP TASK "%s"' % self.func_name)
                break
            logger.debug('NEXT TASK "%s" %s' % (self.func_name, delta))
            await asyncio.sleep(int(delta.total_seconds()))
            logger.info('RUN TASK "%s"' % self.func_name)
            try:
                ret = self.func(app)
                if inspect.isawaitable(ret):
                    await ret
                logger.info('END TASK "%s"' % self.func_name)
            except Exception as e:
                logger.error(e)
                logger.error(traceback.format_exc())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : test
# @Time         : 2020-03-21 17:15
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :
""":argument
zk配置接口 http://0.0.0.0:8000/app/cfg
email post 接口: http://0.0.0.0:8000/app/{biz}  {'msg': 'xx'}
mongo查询get接口:
    http://0.0.0.0:8000/app/mongo/test/count_documents?filter={}
    http://tql.algo.browser.miui.srv/app/mongo/word2vec.tag2vec/find?filter={%27w%27:%20%27%E4%B8%AD%E5%9B%BD%27}

mongo插数post接口 http://0.0.0.0:8000/app/mongo/post/test?jobid=xxxxxxxxxx&jobtype=hive
数据工厂http回调post form接口 http://0.0.0.0:8000/app/mongo/form/test?jobid=xxxxxxxxxx&jobtype=hive


"""

import uvicorn
from mi.app import mongo

app = mongo.app
uvicorn.run(app, host='0.0.0.0', port=8000, workers=1, debug=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : fds2hdfs
# @Time         : 2020/11/20 5:20 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

# 上传数据: ls -a && rm .data.tsv.crc
logger.info("上传数据")
hdfs = "/fds/infra-client/bin/hdfs --cluster zjyprc-hadoop dfs"
os.system(f"rm .*.crc && touch _SUCCESS")
os.system("kinit -k -t /fds/data/h_browser.keytab h_browser@XIAOMI.HADOOP && klist")
logger.info(os.popen(f"{hdfs} -ls").read())
os.system(f"{hdfs} -put -f _SUCCESS /user/h_data_platform/platform/browser/content_daily/date=20201119")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : hdfs2fds
# @Time         : 2020/11/20 2:25 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *

from meutils.io_utils import tf_read_text

import tensorflow as tf

data = tf_read_text(tf, "/user/h_data_platform/work/kolibre/output/20201120/adhoc_194518/*")

for i in data:
    print(i.numpy().decode(), '########')
    break
data = list(map(lambda x: x.numpy().decode().split('\t'), data))
df = pd.DataFrame(data[1:], columns=data[0])
df.to_csv('/fds/1_Work/5_ocr/data2.txt', '\t', index=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : hdfs_test
# @Time         : 2020/11/19 6:13 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import pandas as pd
import tensorflow as tf


# def tf_read_text(
#         pattern,
#         file='/content_daily/date=20201117/000000_0',
#         prefix='hdfs://zjyprc-hadoop/user/h_data_platform/browser'
# ):
#     pattern = prefix + file
#
#     fs = tf.data.Dataset.list_files(file_pattern=pattern)
#     # ds = tf.data.TFRecordDataset(filenames=fs)
#     data = tf.data.TextLineDataset(fs)
#     column_names = ['id', 'category', 'source', 'title', 'url']
#     df = pd.DataFrame([i.numpy().decode().split('\t') for i in data], columns=column_names)
#     print("#####")
#     print(df.head())

# #  Quote inside a string has to be escaped by another quote
d = tf.data.experimental.make_csv_dataset(
    'hdfs://zjyprc-hadoop/user/h_data_platform/platform/browser/browser_intermediate/data/o2o_title_add_daily/date=20201119/part*',
    batch_size=1,
    field_delim='\t',
    num_epochs=1,
    header=True,
    shuffle=False,
    use_quote_delim=False

)
#
df = pd.DataFrame(d.as_numpy_iterator())

print("#########")

print(df.head(10))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : xindao
# @Time         : 2020/9/1 2:57 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from tqdm.auto import tqdm

tqdm.pandas()

from tql.data_plot import WordCloud
from tql.pipe import *
from pathlib import Path
import cufflinks as cf
from tql.algo_nlp.utils import TextClean
from mi.db import Mysql

m = Mysql(
    host='10.136.128.113',
    port=6071,
    user='browser_c3_s',
    passwd='8d8b9339efef09794c8ea2267589b2bc',
    db='browser',
)
sql = """
select *
from xindao_subject2comment 
where comment_videos is not null 
and comment_videos != "" 
and comment_style='video'
and comment_videos like '%url%'
and comment_source = "bilibili" 
and comment_final_id is not null
and comment_article_title!=''
"""
df = pd.read_sql(sql, m.conn)

df = (
    df.assign(origin_features=lambda df: df['origin_features'].map(json.loads))
        .assign(OriginCat=lambda df: df['origin_features'].map(lambda x: x['OriginCat']))
        .assign(OriginSubCat=lambda df: df['origin_features'].map(lambda x: x['OriginSubCat']))

)

tc = TextClean()


def create_word_cloud(series: pd.Series):
    wc = {}
    new_col = []
    for s in tqdm(series):
        ws = list(tc.get_noun(s))
        new_col.append(' '.join(ws))
        for w in ws:
            wc[w] = wc.get(w, 0) + 1

    WordCloud(wc.items()).wc.render(f"WordCloud_{series.name}.html")

    wc_ = pd.DataFrame(wc.items(), columns=['word', 'frequency'])
    df_noun = series.to_frame().assign(noun=new_col)

    return df_noun, wc_



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : demo
# @Time         : 2020-02-14 11:52
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :
# https://github.com/milvus-io/docs/blob/master/site/en/guides/milvus_operation.md#createdrop-indexes-in-a-collection
# https://github.com/milvus-io/pymilvus/tree/master/examples/indexes
# https://raw.githubusercontent.com/milvus-io/pymilvus/0.7.0/examples/example.py
# TODO:
# m.milvus.preload_table


import numpy as np
from milvus import Milvus

"""
client.count_entities

client.create_collection
client.create_index
client.create_partition

client.get_collection_info
client.get_collection_stats
client.get_config
client.get_entity_by_id

client.list_id_in_segment
client.list_partitions

client.delete_entity_by_id
client.drop_collection
client.drop_index
client.drop_partition

client.load_collection???

"""


class ANN(object):

    def __init__(self, host='10.119.33.90', port='19530'):
        self.client = Milvus(host, port)

        print("Client Version:", self.client.client_version())
        print("Server Version:", self.client.server_version())

        self.collections = self.client.list_collections()

    def create_collection(self, collection_name, fields, partition_tag=None, overwrite=True):
        """

        :param collection_name:
        :param fields:
            collection_param = {
                "fields": [
                    #  Milvus doesn't support string type now, but we are considering supporting it soon.
                    #  {"name": "title", "type": DataType.STRING},
                    {"name": "category_", "type": DataType.INT32},
                    {"name": "vector", "type": DataType.FLOAT_VECTOR, "params": {"dim": 768}},
                ],
                "segment_row_limit": 4096,
                "auto_id": False
            }

        :param overwrite:
        :return:
        """
        if self.client.has_collection(collection_name) and overwrite:
            self.client.drop_collection(collection_name)
            self.client.create_collection(collection_name, fields)
        elif self.client.has_collection(collection_name):
            print(f"{collection_name} already exist !!!")
        else:
            self.client.create_collection(collection_name, fields)

        if partition_tag is not None:
            self.client.create_partition(collection_name, partition_tag=partition_tag)

    def create_index(self, collection_name, field_name, index_type='IVF_FLAT', metric_type='IP', index_params=None):
        """
        MetricType:
            INVALID = 0
            L2 = 1
            IP = 2
            # Only supported for byte vectors
            HAMMING = 3
            JACCARD = 4
            TANIMOTO = 5
            #
            SUBSTRUCTURE = 6
            SUPERSTRUCTURE = 7
        IndexType:
            INVALID = 0
            FLAT = 1
            IVFLAT = 2
            IVF_SQ8 = 3
            RNSG = 4
            IVF_SQ8H = 5
            IVF_PQ = 6
            HNSW = 11
            ANNOY = 12

            # alternative name
            IVF_FLAT = IVFLAT
            IVF_SQ8_H = IVF_SQ8H

        class DataType(IntEnum):
            NULL = 0
            INT8 = 1
            INT16 = 2
            INT32 = 3
            INT64 = 4

            STRING = 20

            BOOL = 30

            FLOAT = 40
            DOUBLE = 41

            VECTOR = 100
            UNKNOWN = 9999

        class RangeType(IntEnum):
            LT = 0   # less than
            LTE = 1  # less than or equal
            EQ = 2   # equal
            GT = 3   # greater than
            GTE = 4  # greater than or equal
            NE = 5   # not equal
        :return:
        """
        if index_params is None:
            index_params = {'nlist': 1024}

        params = {
            'index_type': index_type,
            # 'index_file_size': 1024,
            'params': index_params,

            'metric_type': metric_type,
        }
        self.client.create_index(collection_name, field_name, params) # field_name='embedding'

    def insert(self):

        self.client.flush()
        pass

    def search(self):
        pass

    def drop_collection(self, collection_name):
        if self.client.has_collection(collection_name):
            self.client.drop_collection(collection_name)

    def drop_partition(self, collection_name, partition_tag):
        if self.client.has_partition(collection_name, partition_tag):
            self.client.drop_partition(collection_name, partition_tag, timeout=30)

    def noramlize(self, x):
        if len(x.shape) > 1:
            return x / np.clip(x ** 2, 1e-12, None).sum(axis=1).reshape((-1, 1) + x.shape[2:]) ** 0.5
        else:
            return x / np.clip(x ** 2, 1e-12, None).sum() ** 0.5



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : BaiduApi
# @Time         : 2020/9/9 10:26 上午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

import random

from functools import lru_cache
from aip import AipOcr
from kazoo.client import KazooClient


class BaiduApi(object):

    def __init__(self):
        self.keys = self.__get_keys()

    # @lru_cache(512)
    def ocr_basic_general(self, image, options=None):
        """

        :param image:
            import requests
            image = requests.get("https://tva1.sinaimg.cn/large/007S8ZIlly1gik7a734ahj308y02qaac.jpg").content
            image = open('file', 'rb').read()
        :param options:
            options = {}
            options["language_type"] = "CHN_ENG"
            options["detect_direction"] = "true"
            options["detect_language"] = "true"
            options["probability"] = "true"
        :return:
        """
        _ = self._ocr_api().basicGeneral(image, options=options)
        return _

    def _ocr_api(self):
        APP_ID = '11516632'
        API_KEY = 'gAw0HlllOyi9sQcZulvPlmSB'
        SECRET_KEY = 'RlseXLG6SGCxXAYCbliTkCZF7KOxRMZA '

        api = AipOcr(APP_ID, API_KEY, SECRET_KEY)
        return api

    # @property
    # def api(self):
    #     self.key = random.choice(self.keys)
    #     return AipOcr(*key)

    def __get_keys(self):
        zk = KazooClient(hosts="tjwqstaging.zk.hadoop.srv:11000")
        zk.start()
        return eval(zk.get('/tql/baidu_sdk')[0])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : EmailApp
# @Time         : 2020-03-06 09:41
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import time
from datetime import datetime
import pandas as pd
from typing import Optional
from fastapi import FastAPI, Form, Depends, File, UploadFile
from pydantic import BaseModel
from starlette.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.responses import \
    RedirectResponse, FileResponse, HTMLResponse, PlainTextResponse
from starlette.status import *

ROUTE = ""
app = FastAPI(
    debug=True,
    openapi_url=f"{ROUTE}/openapi.json",
    docs_url=f"{ROUTE}/docs",
    redoc_url=f"{ROUTE}/redoc",
    swagger_ui_oauth2_redirect_url=f"{ROUTE}/docs/oauth2-redirect"
)


@app.get("/demo}")
async def report():
    pass

@app.post("/report/{job_type}")
async def report(request: Request, kwargs: dict, job_type):
    """https://www.jianshu.com/p/5b922809f5ef
    一般对于Request Body不会通过get提交，对于get提交的参数一般称为是查询参数。
    所以，如果是通过POTS,PUT等方式提交的参数信息，我们一般是放到Request Body来提交到我们的后端。
    """
    # post: kwargs: dict
    # print(dict(request))
    # print(request.path_params) # {'job_type': 'hive'}
    dic = dict(request.query_params)
    dic['date'] = datetime.now().__str__()
    dic['post_data'] = kwargs
    dic['form_data'] = dict(await request.form())

    print(dic)
    if job_type == 'hive':
        return "post succeed"

    elif job_type == 'spark':
        pass


if __name__ == '__main__':
    import socket
    import uvicorn

    me = socket.gethostname() == 'yuanjie-Mac.local'
    main_file = __file__.split('/')[-1].split('.')[0]

    # app参数必须是个字符串才能开启热更新
    uvicorn.run(app=f"{main_file}:app", host='0.0.0.0', port=8000, workers=1, debug=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : ts_predict
# @Time         : 2020/7/22 12:58 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


def ts_predict(df):
    from fbprophet import Prophet
    from fbprophet.plot import plot_plotly
    import plotly.offline as py
    py.init_notebook_mode()

    m = Prophet()
    m.fit(df)
    future = m.make_future_dataframe(periods=31)
    forecast = m.predict(future)

    fig = plot_plotly(m, forecast)  # This returns a plotly Figure
    py.iplot(fig)
    return forecast


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : Python.
# @File         : mimongo
# @Time         : 2020-03-20 11:26
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :
# https://www.jb51.net/article/159652.htm
# https://www.cnblogs.com/kaituorensheng/p/5181410.html

import pymongo
from pymongo import MongoClient

from tqdm.auto import tqdm
from ..utils import isMac


class Mongo(object):
    """插入速度200维向量 20w/min
    tag2vec = m.db.word2vec.tag2vec
    tag2vec.delete_many({})
    tag2vec.count_documents({})
    docs = get_docs(dff.w, dff.v)
    tag2vec.insert_many(docs, False)
    tag2vec.find_one()
    tag2vec.count_documents({})

    # https://www.cnblogs.com/wangyuxing/p/9879504.html
    news_word2vec.find_one({'id': 0}, projection={'v': 0}) # 指定返回键值
    tag2vec.find({'w': {'$in': ['娱乐', '体育']}}) # $in $nin

    l = [{'w': 1, 'v': 2}, {'w': 3, 'v': 4}]
    dic = dict([i.values() for i in l])
    [{'w': k, 'v': v} for k, v in dic.items()]

    """

    def __init__(self, db='mig3_algo_push', print_info=False):
        """
        10.162.39.41:28026
        mig3_algo_push
        :param db:
        :param print_info:
        """
        database = 'mig3_algo_push'
        user = 'mig3_algo_push_wr'
        passwd = 'zxBy84SkD3G4ce9424S4IB5HzYZ29uGX'
        iplist = '10.162.39.41:28026,10.162.52.42:28026,10.132.57.49:28026'
        replicaSet = 'mig_algo_push'

        url = "mongodb://localhost:27017" if isMac else f"mongodb://{user}:{passwd}@{iplist}/{database}?replicaSet={replicaSet}&authSource=admin"
        self.client = MongoClient(url)
        self.db = self.client[db]
        self.collection_names = self.db.collection_names()

        # Info
        if print_info:
            print({
                "主节点": self.client.is_primary,
                "最大连接数": self.client.max_pool_size
            })
            print(self.client.admin.command('ismaster'))

    def insert(self, collection: pymongo.collection.Collection, docs):
        if isinstance(docs, dict):
            docs = [docs]
        print("插入前documents:", collection.count_documents({}))
        collection.insert_many(docs, False)
        print("插入后documents:", collection.count_documents({}))

    def get_docs(self, keys, values):
        return [{'id': i, 'w': k, 'v': v} for i, (k, v) in tqdm(enumerate(zip(keys, values)))]

    def drop_collection(self, name_or_collection):
        self.db.drop_collection(name_or_collection)

    def __starter(self):
        """
        # info
        collection.count_documents({})

        # 增
        collection.insert_one({'x': 1})
        collection.insert_one({'xx': 2})
        ids = collection.insert_many([{'xxx': 3}, {'xxx': 4}])

        from bson.objectid import ObjectId

        collection.find_one({'x': 1})
        collection.find_one({'_id': ObjectId('5e743cea06be472ac7298def')})

        # 复杂的查询
        list(collection.find({})) # collection.find().count()
        list(collection.find({'xx': {'$gt': 1}}))

        condition = {'s': 1}
        # result = collection.update(condition, {'s': 1, 'ss': 2}) # 完全替换
        result = collection.update(condition, {'$set': {'s':1, 'sss': 3}}) # {**d1, **d2}
        """
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : df2html
# @Time         : 2020-03-04 13:57
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


head = \
    """
    <head>
        <meta charset="utf-8">
        <STYLE TYPE="text/css" MEDIA=screen>

            table.dataframe {
                border-collapse: collapse;
                border: 2px solid #a19da2;
                /*居中显示整个表格*/
                margin: auto;
            }

            table.dataframe thead {
                border: 2px solid #91c6e1;
                background: #f1f1f1;
                padding: 10px 10px 10px 10px;
                color: #333333;
            }

            table.dataframe tbody {
                border: 2px solid #91c6e1;
                padding: 10px 10px 10px 10px;
            }

            table.dataframe tr {

            }

            table.dataframe th {
                vertical-align: top;
                font-size: 14px;
                padding: 10px 10px 10px 10px;
                color: #105de3;
                font-family: arial;
                text-align: center;
            }

            table.dataframe td {
                text-align: center;
                padding: 10px 10px 10px 10px;
            }

            body {
                font-family: 宋体;
            }

            h1 {
                color: #5db446
            }

            div.header h2 {
                color: #0002e3;
                font-family: 黑体;
            }

            div.content h2 {
                text-align: center;
                font-size: 28px;
                text-shadow: 2px 2px 1px #de4040;
                color: #fff;
                font-weight: bold;
                background-color: #008eb7;
                line-height: 1.5;
                margin: 20px 0;
                box-shadow: 10px 10px 5px #888888;
                border-radius: 5px;
            }

            h3 {
                font-size: 22px;
                background-color: rgba(0, 2, 227, 0.71);
                text-shadow: 2px 2px 1px #de4040;
                color: rgba(239, 241, 234, 0.99);
                line-height: 1.5;
            }

            h4 {
                color: #e10092;
                font-family: 楷体;
                font-size: 20px;
                text-align: center;
            }

            td img {
                /*width: 60px;*/
                max-width: 300px;
                max-height: 300px;
            }

        </STYLE>
    </head>
    """

body = """
        <body>

        <div align="center" class="header">
            <!--标题部分的信息-->
            <h1 align="center">{subject}</h1>
            <h2 align="center">{date}</h2>
        </div>

        <hr>

        <div class="content">
            <!--正文内容-->
            <h2> </h2>

            <div>
                <h4></h4>
                {df_html}

        </div>

        <hr>

        <p style="text-align: center">

        </p>
        </body>
        """


def df2html(subject, date, df_html):
    return head + body.format(subject=subject, date=date, df_html=df_html)
